@@ -1,11 +1,11 @@
 /*
- Copyright (c) 2009-2013,
+ Copyright (c) 2009-2014,
  Ken Arroyo Ohori    g.a.k.arroyoohori@tudelft.nl
  Hugo Ledoux         h.ledoux@tudelft.nl
  Martijn Meijers     b.m.meijers@tudelft.nl
  All rights reserved.
  
- // This file is part of prepair: you can redistribute it and/or modify
+ This file is part of prepair: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
@@ -106,146 +106,6 @@ void PolygonRepair::removeSmallPolygons(OGRMultiPolygon *outPolygons, double min
   }
 }
 
-OGRMultiPolygon *PolygonRepair::isr(OGRGeometry *geometry, double tolerance) {
-  
-  // Convert to multipolygon
-  OGRMultiPolygon *multipolygon = new OGRMultiPolygon();
-  switch (geometry->getGeometryType()) {
-    case wkbLineString: {
-      OGRLineString *oldRing = static_cast<OGRLineString *>(geometry);
-      OGRPolygon *polygon = new OGRPolygon();
-      OGRLinearRing *ring = new OGRLinearRing();
-      OGRPoint lastPoint, point;
-      for (int currentPoint = 0; currentPoint < oldRing->getNumPoints(); ++currentPoint) {
-        oldRing->getPoint(currentPoint, &point);
-        if (currentPoint > 0) {
-          if (!point.Equals(&lastPoint)) ring->addPoint(&point);
-        } lastPoint = point;
-      } polygon->addRingDirectly(ring);
-      multipolygon->addGeometryDirectly(polygon);
-      break;
-    }
-      
-    case wkbPolygon: {
-      OGRPolygon *oldPolygon = static_cast<OGRPolygon *>(geometry);
-      OGRPolygon *polygon = new OGRPolygon();
-      OGRLinearRing *oldRing = oldPolygon->getExteriorRing();
-      OGRLinearRing *ring = new OGRLinearRing();
-      OGRPoint lastPoint, point;
-      for (int currentPoint = 0; currentPoint < oldRing->getNumPoints(); ++currentPoint) {
-        oldRing->getPoint(currentPoint, &point);
-        if (currentPoint > 0) {
-          if (!point.Equals(&lastPoint)) ring->addPoint(&point);
-        } lastPoint = point;
-      } polygon->addRingDirectly(ring);
-      for (int currentRing = 0; currentRing < oldPolygon->getNumInteriorRings(); ++currentRing) {
-        oldRing = oldPolygon->getInteriorRing(currentRing);
-        ring = new OGRLinearRing();
-        for (int currentPoint = 0; currentPoint < oldRing->getNumPoints(); ++currentPoint) {
-          oldRing->getPoint(currentPoint, &point);
-          if (currentPoint > 0) {
-            if (!point.Equals(&lastPoint)) ring->addPoint(&point);
-          } lastPoint = point;
-        } polygon->addRingDirectly(ring);
-      } multipolygon->addGeometryDirectly(polygon);
-      break;
-    }
-      
-    case wkbMultiPolygon: {
-      OGRMultiPolygon *oldMultipolygon = static_cast<OGRMultiPolygon *>(geometry);
-      OGRPolygon *oldPolygon, *polygon;
-      OGRLinearRing *oldRing, *ring;
-      OGRPoint lastPoint, point;
-      for (int currentPolygon = 0; currentPolygon < oldMultipolygon->getNumGeometries(); ++currentPolygon) {
-        oldPolygon = static_cast<OGRPolygon *>(oldMultipolygon->getGeometryRef(currentPolygon));
-        polygon = new OGRPolygon();
-        oldRing = oldPolygon->getExteriorRing();
-        ring = new OGRLinearRing();
-        for (int currentPoint = 0; currentPoint < oldRing->getNumPoints(); ++currentPoint) {
-          oldRing->getPoint(currentPoint, &point);
-          if (currentPoint > 0) {
-            if (!point.Equals(&lastPoint)) ring->addPoint(&point);
-          } lastPoint = point;
-        } polygon->addRingDirectly(ring);
-        for (int currentRing = 0; currentRing < oldPolygon->getNumInteriorRings(); ++currentRing) {
-          oldRing = oldPolygon->getInteriorRing(currentRing);
-          ring = new OGRLinearRing();
-          for (int currentPoint = 0; currentPoint < oldRing->getNumPoints(); ++currentPoint) {
-            oldRing->getPoint(currentPoint, &point);
-            if (currentPoint > 0) {
-              if (!point.Equals(&lastPoint)) ring->addPoint(&point);
-            } lastPoint = point;
-          } polygon->addRingDirectly(ring);
-        } multipolygon->addGeometryDirectly(polygon);
-      } break;
-    }
-      
-    default:
-      std::cerr << "PolygonRepair::isr: Cannot understand input." << std::endl;
-      return new OGRMultiPolygon();
-      break;
-  } multipolygon->closeRings();
-  
-  // Put all line segments in a list (in order)
-  // TODO: Only do point location once for segment.
-  std::list<Segment> inputSegments;
-  for (int currentPolygon = 0; currentPolygon < multipolygon->getNumGeometries(); ++currentPolygon) {
-    OGRPolygon *polygon = static_cast<OGRPolygon *>(multipolygon->getGeometryRef(currentPolygon));
-    for (int currentPoint = 1; currentPoint < polygon->getExteriorRing()->getNumPoints(); ++currentPoint) {
-      Segment thisSegment(Point(polygon->getExteriorRing()->getX(currentPoint-1), polygon->getExteriorRing()->getY(currentPoint-1)),
-                          Point(polygon->getExteriorRing()->getX(currentPoint), polygon->getExteriorRing()->getY(currentPoint)));
-      if (thisSegment.is_degenerate()) {
-        std::cout << "This should not happen!!!" << std::endl;
-        return new OGRMultiPolygon();
-      } inputSegments.push_back(thisSegment);
-    } for (int currentRing = 0; currentRing < polygon->getNumInteriorRings(); ++currentRing) {
-      for (int currentPoint = 1; currentPoint < polygon->getInteriorRing(currentRing)->getNumPoints(); ++currentPoint) {
-        Segment thisSegment(Point(polygon->getExteriorRing()->getX(currentPoint-1), polygon->getExteriorRing()->getY(currentPoint-1)),
-                            Point(polygon->getExteriorRing()->getX(currentPoint), polygon->getExteriorRing()->getY(currentPoint)));
-        if (thisSegment.is_degenerate()) {
-          std::cout << "This should not happen!!!" << std::endl;
-          return new OGRMultiPolygon();
-        } std::cout << thisSegment << std::endl;
-        inputSegments.push_back(thisSegment);
-      }
-    }
-  }
-  
-  // Snap
-  std::list<std::list<Point> > outputPolylines;
-  CGAL::snap_rounding_2<SRT, std::list<Segment>::const_iterator, std::list<std::list<Point> > >(inputSegments.begin(), inputSegments.end(), outputPolylines, tolerance, true, false, 2);
-  
-  // Put it back into an OGRMultiPolygon
-  OGRMultiPolygon *snappedMultipolygon = new OGRMultiPolygon();
-  std::list<std::list<Point> >::iterator currentPolyline = outputPolylines.begin();
-  for (int currentPolygon = 0; currentPolygon < multipolygon->getNumGeometries(); ++currentPolygon) {
-    OGRPolygon *polygon = static_cast<OGRPolygon *>(multipolygon->getGeometryRef(currentPolygon));
-    OGRPolygon *snappedPolygon = new OGRPolygon();
-    OGRLinearRing *snappedOuterRing = new OGRLinearRing();
-    for (int currentPoint = 1; currentPoint < polygon->getExteriorRing()->getNumPoints(); ++currentPoint) {
-      std::list<Point>::iterator point = currentPolyline->begin();
-      ++point;
-      while (point != currentPolyline->end()) {
-        snappedOuterRing->addPoint(CGAL::to_double(point->x()), CGAL::to_double(point->y()));
-        ++point;
-      } ++currentPolyline;
-    } snappedPolygon->addRingDirectly(snappedOuterRing);
-    for (int currentRing = 0; currentRing < polygon->getNumInteriorRings(); ++currentRing) {
-      OGRLinearRing *snappedInnerRing = new OGRLinearRing();
-      for (int currentPoint = 1; currentPoint < polygon->getInteriorRing(currentRing)->getNumPoints(); ++currentPoint) {
-        std::list<Point>::iterator point = currentPolyline->begin();
-        ++point;
-        while (point != currentPolyline->end()) {
-          snappedOuterRing->addPoint(CGAL::to_double(point->x()), CGAL::to_double(point->y()));
-          ++point;
-        } ++currentPolyline;
-      } snappedPolygon->addRingDirectly(snappedInnerRing);
-    } snappedMultipolygon->OGRGeometryCollection::addGeometryDirectly(snappedPolygon);
-  }
-  
-  return snappedMultipolygon;
-}
-
 double PolygonRepair::computeRobustness(OGRGeometry *geometry) {
   double smallestdist = 1e99;
   
@@ -305,22 +165,44 @@ double PolygonRepair::computeRobustness(OGRGeometry *geometry) {
 
 bool PolygonRepair::saveToShp(OGRGeometry* geometry, const char *fileName) {
   const char *driverName = "ESRI Shapefile";
+#if GDAL_VERSION_MAJOR < 2
   OGRRegisterAll();
 	OGRSFDriver *driver = OGRSFDriverRegistrar::GetRegistrar()->GetDriverByName(driverName);
+#else
+  GDALAllRegister();
+	GDALDriver *driver = GetGDALDriverManager()->GetDriverByName(driverName);
+#endif
 	if (driver == NULL) {
 		std::cout << "\tError: OGR Shapefile driver not found." << std::endl;
 		return false;
 	}
+
+#if GDAL_VERSION_MAJOR < 2
 	OGRDataSource *dataSource = driver->Open(fileName, false);
+#else
+	GDALDataset *dataSource = (GDALDataset*) GDALOpenEx(fileName, GDAL_OF_READONLY, NULL, NULL, NULL);
+#endif
 	if (dataSource != NULL) {
 		std::cout << "\tOverwriting file..." << std::endl;
+#if GDAL_VERSION_MAJOR < 2
 		if (driver->DeleteDataSource(dataSource->GetName())!= OGRERR_NONE) {
+#else
+		if (driver->Delete(fileName)!= CE_None) {
+#endif
 			std::cout << "\tError: Couldn't erase file with same name." << std::endl;
 			return false;
+#if GDAL_VERSION_MAJOR < 2
 		} OGRDataSource::DestroyDataSource(dataSource);
+#else
+		} GDALClose(dataSource);
+#endif
 	}
 	std::cout << "\tCreating " << fileName << std::endl;
+#if GDAL_VERSION_MAJOR < 2
 	dataSource = driver->CreateDataSource(fileName, NULL);
+#else
+	dataSource = driver->Create(fileName,0,0,0,GDT_Unknown,NULL);
+#endif
 	if (dataSource == NULL) {
 		std::cout << "\tError: Could not create file." << std::endl;
 		return false;
@@ -337,7 +219,11 @@ bool PolygonRepair::saveToShp(OGRGeometry* geometry, const char *fileName) {
     std::cout << "\tError: Could not create feature." << std::endl;
   }
   OGRFeature::DestroyFeature(feature);
+#if GDAL_VERSION_MAJOR < 2
   OGRDataSource::DestroyDataSource(dataSource);
+#else
+  GDALClose(dataSource);
+#endif
   return true;
 }
 
@@ -358,6 +244,7 @@ void PolygonRepair::insertConstraints(Triangulation &triangulation, OGRGeometry*
         vb = triangulation.insert(Point(ring->getX(currentPoint),
                                         ring->getY(currentPoint)),
                                   triangulation.incident_faces(va));
+        if (va == vb) continue;
         if (removeOverlappingConstraints && triangulation.is_edge(va, vb, faceOfEdge, indexOfEdge)) {
           if (triangulation.is_constrained(std::pair<Triangulation::Face_handle, int>(faceOfEdge, indexOfEdge))) {
             triangulation.insert_constraint(va, vb); // trick to remove a partially overlapping constraint
@@ -371,12 +258,14 @@ void PolygonRepair::insertConstraints(Triangulation &triangulation, OGRGeometry*
       OGRPolygon *polygon = static_cast<OGRPolygon *>(geometry);
       
       // Outer
+      polygon->getExteriorRing()->closeRings();
       vb = triangulation.insert(Point(polygon->getExteriorRing()->getX(0), polygon->getExteriorRing()->getY(0)));
       for (int currentPoint = 1; currentPoint < polygon->getExteriorRing()->getNumPoints(); ++currentPoint) {
         va = vb;
         vb = triangulation.insert(Point(polygon->getExteriorRing()->getX(currentPoint),
                                         polygon->getExteriorRing()->getY(currentPoint)),
                                   triangulation.incident_faces(va));
+        if (va == vb) continue;
         if (removeOverlappingConstraints && triangulation.is_edge(va, vb, faceOfEdge, indexOfEdge)) {
           if (triangulation.is_constrained(std::pair<Triangulation::Face_handle, int>(faceOfEdge, indexOfEdge))) {
             triangulation.insert_constraint(va, vb);
@@ -394,6 +283,7 @@ void PolygonRepair::insertConstraints(Triangulation &triangulation, OGRGeometry*
       
       // Inner
       for (int currentRing = 0; currentRing < polygon->getNumInteriorRings(); ++currentRing) {
+        polygon->getInteriorRing(currentRing)->closeRings();
         vb = triangulation.insert(Point(polygon->getInteriorRing(currentRing)->getX(0),
                                         polygon->getInteriorRing(currentRing)->getY(0)));
         for (int currentPoint = 1; currentPoint < polygon->getInteriorRing(currentRing)->getNumPoints(); ++currentPoint) {
@@ -401,6 +291,7 @@ void PolygonRepair::insertConstraints(Triangulation &triangulation, OGRGeometry*
           vb = triangulation.insert(Point(polygon->getInteriorRing(currentRing)->getX(currentPoint),
                                           polygon->getInteriorRing(currentRing)->getY(currentPoint)),
                                     triangulation.incident_faces(va));
+          if (va == vb) continue;
           if (removeOverlappingConstraints && triangulation.is_edge(va, vb, faceOfEdge, indexOfEdge)) {
             if (triangulation.is_constrained(std::pair<Triangulation::Face_handle, int>(faceOfEdge, indexOfEdge))) {
               triangulation.insert_constraint(va, vb);
@@ -425,6 +316,7 @@ void PolygonRepair::insertConstraints(Triangulation &triangulation, OGRGeometry*
         OGRPolygon *polygon = static_cast<OGRPolygon *>(multipolygon->getGeometryRef(currentPolygon));
         
         // Outer
+        polygon->getExteriorRing()->closeRings();
         vb = triangulation.insert(Point(polygon->getExteriorRing()->getX(0),
                                         polygon->getExteriorRing()->getY(0)));
         for (int currentPoint = 1; currentPoint < polygon->getExteriorRing()->getNumPoints(); ++currentPoint) {
@@ -432,6 +324,7 @@ void PolygonRepair::insertConstraints(Triangulation &triangulation, OGRGeometry*
           vb = triangulation.insert(Point(polygon->getExteriorRing()->getX(currentPoint),
                                           polygon->getExteriorRing()->getY(currentPoint)),
                                     triangulation.incident_faces(va));
+          if (va == vb) continue;
           if (removeOverlappingConstraints && triangulation.is_edge(va, vb, faceOfEdge, indexOfEdge)) {
             if (triangulation.is_constrained(std::pair<Triangulation::Face_handle, int>(faceOfEdge, indexOfEdge))) {
               triangulation.insert_constraint(va, vb);
@@ -442,6 +335,7 @@ void PolygonRepair::insertConstraints(Triangulation &triangulation, OGRGeometry*
         
         // Inner
         for (int currentRing = 0; currentRing < polygon->getNumInteriorRings(); ++currentRing) {
+          polygon->getInteriorRing(currentRing)->closeRings();
           vb = triangulation.insert(Point(polygon->getInteriorRing(currentRing)->getX(0),
                                           polygon->getInteriorRing(currentRing)->getY(0)));
           for (int currentPoint = 1; currentPoint < polygon->getInteriorRing(currentRing)->getNumPoints(); ++currentPoint) {
@@ -449,6 +343,7 @@ void PolygonRepair::insertConstraints(Triangulation &triangulation, OGRGeometry*
             vb = triangulation.insert(Point(polygon->getInteriorRing(currentRing)->getX(currentPoint),
                                             polygon->getInteriorRing(currentRing)->getY(currentPoint)),
                                       triangulation.incident_faces(va));
+            if (va == vb) continue;
             if (removeOverlappingConstraints && triangulation.is_edge(va, vb, faceOfEdge, indexOfEdge)) {
               if (triangulation.is_constrained(std::pair<Triangulation::Face_handle, int>(faceOfEdge, indexOfEdge))) {
                 triangulation.insert_constraint(va, vb);
