@@ -1,11 +1,11 @@
 /*
- Copyright (c) 2009-2013,
+ Copyright (c) 2009-2014,
  Ken Arroyo Ohori    g.a.k.arroyoohori@tudelft.nl
  Hugo Ledoux         h.ledoux@tudelft.nl
  Martijn Meijers     b.m.meijers@tudelft.nl
  All rights reserved.
  
- // This file is part of prepair: you can redistribute it and/or modify
+ This file is part of prepair: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
@@ -22,6 +22,8 @@
 #ifndef TRIANGLEINFO_H
 #define TRIANGLEINFO_H
 
+#include <bitset>
+
 // TODO: Maybe create a marker manager in the future
 class TriangleInfo {
 public:
@@ -38,8 +40,7 @@ public:
   void beenReconstructed(bool reconstructed);
   
 private:
-  unsigned char info; // Bitset: unused                          unused        unused            unused
-                      //         not reconstructed/reconstructed normal/border exterior/interior untagged/tagged
+  std::bitset<4> info;
 };
 
 #endif
